@@ -36,6 +36,7 @@ struct op {
   unsigned int addr;
   unsigned int width;			/* Byte width of the access */
   unsigned int num_values;		/* Number of values to write; 0=read */
+  unsigned int number;                 /* The n-th capability of that id */
   struct value values[0];
 };
 
@@ -84,11 +85,20 @@ exec_op(struct op *op, struct pci_dev *dev)
   if (op->cap_type)
     {
       struct pci_cap *cap;
-      cap = pci_find_cap(dev, op->cap_id, op->cap_type);
+      unsigned int cap_nr = op->number;
+      cap = pci_find_cap_nr(dev, op->cap_id, op->cap_type, &cap_nr);
       if (cap)
-	addr = cap->addr;
+        addr = cap->addr;
+      else if (cap_nr == 0)
+        die("%s: Instance #%d of %s %04x not found - there are no capabilities with that id.", slot,
+            op->number, ((op->cap_type == PCI_CAP_NORMAL) ? "Capability" : "Extended capability"),
+            op->cap_id);
       else
-	die("%s: %s %04x not found", slot, ((op->cap_type == PCI_CAP_NORMAL) ? "Capability" : "Extended capability"), op->cap_id);
+        die("%s: Instance #%d of %s %04x not found - there %s only %d %s with that id.", slot,
+            op->number, ((op->cap_type == PCI_CAP_NORMAL) ? "Capability" : "Extended capability"),
+            op->cap_id, ((cap_nr == 1) ? "is" : "are"), cap_nr,
+            ((cap_nr == 1) ? "capability" : "capabilities"));
+
       trace(((op->cap_type == PCI_CAP_NORMAL) ? "(cap %02x @%02x) " : "(ecap %04x @%03x) "), op->cap_id, addr);
     }
   addr += op->addr;
@@ -287,6 +297,7 @@ static const struct reg_name pci_reg_names[] = {
   { 0x10011,    0, 0, "CAP_MSIX" },
   { 0x10012,    0, 0, "CAP_SATA" },
   { 0x10013,    0, 0, "CAP_AF" },
+  { 0x10014,    0, 0, "CAP_EA" },
   { 0x20001,	0, 0, "ECAP_AER" },
   { 0x20002,	0, 0, "ECAP_VC" },
   { 0x20003,	0, 0, "ECAP_DSN" },
@@ -295,13 +306,37 @@ static const struct reg_name pci_reg_names[] = {
   { 0x20006,	0, 0, "ECAP_RCILINK" },
   { 0x20007,	0, 0, "ECAP_RCECOLL" },
   { 0x20008,	0, 0, "ECAP_MFVC" },
+  { 0x20009,	0, 0, "ECAP_VC2" },
   { 0x2000a,	0, 0, "ECAP_RBCB" },
   { 0x2000b,	0, 0, "ECAP_VNDR" },
   { 0x2000d,	0, 0, "ECAP_ACS" },
   { 0x2000e,	0, 0, "ECAP_ARI" },
   { 0x2000f,	0, 0, "ECAP_ATS" },
   { 0x20010,	0, 0, "ECAP_SRIOV" },
+  { 0x20011,	0, 0, "ECAP_MRIOV" },
+  { 0x20012,	0, 0, "ECAP_MCAST" },
+  { 0x20013,	0, 0, "ECAP_PRI" },
+  { 0x20015,	0, 0, "ECAP_REBAR" },
+  { 0x20016,	0, 0, "ECAP_DPA" },
+  { 0x20017,	0, 0, "ECAP_TPH" },
+  { 0x20018,	0, 0, "ECAP_LTR" },
+  { 0x20019,	0, 0, "ECAP_SECPCI" },
+  { 0x2001a,	0, 0, "ECAP_PMUX" },
+  { 0x2001b,	0, 0, "ECAP_PASID" },
+  { 0x2001c,	0, 0, "ECAP_LNR" },
   { 0x2001d,	0, 0, "ECAP_DPC" },
+  { 0x2001e,	0, 0, "ECAP_L1PM" },
+  { 0x2001f,	0, 0, "ECAP_PTM" },
+  { 0x20020,	0, 0, "ECAP_M_PCIE" },
+  { 0x20021,	0, 0, "ECAP_FRS" },
+  { 0x20022,	0, 0, "ECAP_RTR" },
+  { 0x20023,	0, 0, "ECAP_DVSEC" },
+  { 0x20024,	0, 0, "ECAP_VF_REBAR" },
+  { 0x20025,	0, 0, "ECAP_DLNK" },
+  { 0x20026,	0, 0, "ECAP_16GT" },
+  { 0x20027,	0, 0, "ECAP_LMR" },
+  { 0x20028,	0, 0, "ECAP_HIER_ID" },
+  { 0x20029,	0, 0, "ECAP_NPEM" },
   {       0,    0, 0, NULL }
 };
 
@@ -341,7 +376,7 @@ GENERIC_HELP
 "Setting commands:\n"
 "<device>:\t-s [[[<domain>]:][<bus>]:][<slot>][.[<func>]]\n"
 "\t\t-d [<vendor>]:[<device>]\n"
-"<reg>:\t\t<base>[+<offset>][.(B|W|L)]\n"
+"<reg>:\t\t<base>[+<offset>][.(B|W|L)][@<number>]\n"
 "<base>:\t\t<address>\n"
 "\t\t<named-register>\n"
 "\t\t[E]CAP_<capability-name>\n"
@@ -557,7 +592,7 @@ static void parse_register(struct op *op, char *base)
 
 static void parse_op(char *c, struct pci_dev **selected_devices)
 {
-  char *base, *offset, *width, *value;
+  char *base, *offset, *width, *value, *number;
   char *e, *f;
   int n, j;
   struct op *op;
@@ -566,6 +601,8 @@ static void parse_op(char *c, struct pci_dev **selected_devices)
   base = xstrdup(c);
   if (value = strchr(base, '='))
     *value++ = 0;
+  if (number = strchr(base, '@'))
+    *number++ = 0;
   if (width = strchr(base, '.'))
     *width++ = 0;
   if (offset = strchr(base, '+'))
@@ -608,6 +645,18 @@ static void parse_op(char *c, struct pci_dev **selected_devices)
   else
     op->width = 0;
 
+  /* Check which n-th capability of the same id we want */
+  if (number)
+    {
+      unsigned int num;
+      if (parse_x32(number, NULL, &num) <= 0 || (int) num < 0)
+          parse_err("Invalid number \"%s\"", number);
+      op->number = num;
+
+    }
+  else
+      op->number = 0;
+
   /* Find the register */
   parse_register(op, base);
   if (!op->width)
